@@ -3,11 +3,11 @@
  * Script to show the differences between the currently saved and the newly
  * generated configuration.
  *
- * $Horde: horde/admin/setup/diff.php,v 1.4.10.6 2007/01/02 13:54:04 jan Exp $
+ * $Horde: horde/admin/setup/diff.php,v 1.4.10.9 2009/01/06 15:22:11 jan Exp $
  *
- * Copyright 2004-2007 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  */
 
@@ -27,7 +27,7 @@ if (!Auth::isAdmin()) {
 $render_type = Util::getFormData('render', 'inline');
 include_once 'Text/Diff/Renderer/' . $render_type . '.php';
 $class = 'Text_Diff_Renderer_' . $render_type;
-$renderer = &new $class();
+$renderer = new $class();
 
 /**
  * Private function to render the differences for a specific app.
@@ -42,8 +42,8 @@ function _getDiff($app)
     $current_config = @file_get_contents($path . '/conf.php');
 
     /* Calculate the differences. */
-    $diff = &new Text_Diff(explode("\n", $current_config),
-                           explode("\n", $_SESSION['_config'][$app]));
+    $diff = new Text_Diff(explode("\n", $current_config),
+                          explode("\n", $_SESSION['_config'][$app]));
     $diff = $renderer->render($diff);
     if (!empty($diff)) {
         return $diff;
@@ -86,7 +86,7 @@ if (!empty($_SESSION['_config'])) {
 }
 
 /* Set up the template. */
-$template = &new Horde_Template();
+$template = new Horde_Template();
 $template->setOption('gettext', true);
 $template->set('diffs', $diffs, true);
 
