@@ -14,12 +14,12 @@
  *                  cookie.</pre>
  *
  *
- * $Horde: framework/Auth/Auth/auto.php,v 1.12.4.9 2007/01/02 13:54:06 jan Exp $
+ * $Horde: framework/Auth/Auth/auto.php,v 1.12.4.13 2009/01/06 15:22:49 jan Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @since   Horde 2.2
@@ -76,10 +76,9 @@ class Auth_auto extends Auth {
         $username = (!empty($this->_params['requestuser']) && isset($_REQUEST['username'])) ?
             $_REQUEST['username'] :
             $this->_params['username'];
-        $this->setAuth($username,
-                       array('transparent' => 1,
-                             'password' => isset($this->_params['password']) ? $this->_params['password'] : null));
-        return true;
+        return $this->setAuth($username,
+            array('transparent' => 1,
+                  'password' => isset($this->_params['password']) ? $this->_params['password'] : null));
     }
 
 }
