@@ -19,12 +19,12 @@
  *                  DEFAULT: NONE (system default will be used)</pre>
  *
  *
- * $Horde: framework/Auth/Auth/ldap.php,v 1.47.10.21 2007/01/02 13:54:07 jan Exp $
+ * $Horde: framework/Auth/Auth/ldap.php,v 1.47.10.33 2009/10/26 11:58:59 jan Exp $
  *
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you did
+ * not receive this file, see http://opensource.org/licenses/lgpl-license.php.
  *
  * @author  Jon Parise <jon@horde.org>
  * @since   Horde 1.3
@@ -58,22 +58,6 @@ class Auth_ldap extends Auth {
      * @param array $params  A hash containing connection parameters.
      */
     function Auth_ldap($params = array())
-    {
-        if (!Util::extensionExists('ldap')) {
-            Horde::fatal(_("Auth_ldap: Required LDAP extension not found."), __FILE__, __LINE__);
-        }
-
-        $this->_setParams($params);
-    }
-
-    /**
-     * Set configuration parameters
-     *
-     * @access private
-     *
-     * @param array $params  A hash containing connection parameters.
-     */
-    function _setParams($params)
     {
         /* Ensure we've been provided with all of the necessary parameters. */
         Horde::assertDriverConfig($params, 'auth',
@@ -92,6 +76,10 @@ class Auth_ldap extends Auth {
      */
     function _connect()
     {
+        if (!Util::extensionExists('ldap')) {
+            return PEAR::raiseError(_("Auth_ldap: Required LDAP extension not found."));
+        }
+
         /* Connect to the LDAP server. */
         $this->_ds = @ldap_connect($this->_params['hostspec']);
         if (!$this->_ds) {
@@ -101,11 +89,34 @@ class Auth_ldap extends Auth {
         if (isset($this->_params['version'])) {
             if (!ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION,
                                  $this->_params['version'])) {
-                Horde::logMessage(sprintf('Set LDAP protocol version to %d failed: [%d] %s',
-                                          $this->_params['version'],
-                                          ldap_errno($conn),
-                                          ldap_error($conn)),
-                                  __FILE__, __LINE__, PEAR_LOG_ERR);
+                Horde::logMessage(
+                    sprintf('Set LDAP protocol version to %d failed: [%d] %s',
+                            $this->_params['version'],
+                            @ldap_errno($this->_ds),
+                            @ldap_error($this->_ds)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+        }
+
+        /* Start TLS if we're using it. */
+        if (!empty($this->_params['tls'])) {
+            if (!@ldap_start_tls($this->_ds)) {
+                Horde::logMessage(
+                    sprintf('STARTTLS failed: [%d] %s',
+                            @ldap_errno($this->_ds),
+                            @ldap_error($this->_ds)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+        }
+
+        /* Work around Active Directory quirk. */
+        if (!empty($this->_params['ad'])) {
+            if (!ldap_set_option($this->_ds, LDAP_OPT_REFERRALS, false)) {
+                Horde::logMessage(
+                    sprintf('Unable to disable directory referrals on this connection to Active Directory: [%d] %s',
+                            @ldap_errno($this->_ds),
+                            @ldap_error($this->_ds)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
             }
         }
 
@@ -135,9 +146,18 @@ class Auth_ldap extends Auth {
     function _findDN($userId)
     {
         /* Search for the user's full DN. */
-        $search = @ldap_search($this->_ds, $this->_params['basedn'],
-                               $this->_params['uid'] . '=' . $userId,
-                               array($this->_params['uid']));
+        $filter = $this->_getParamFilter();
+        $filter = '(&(' . $this->_params['uid'] . '=' . $userId . ')' .
+                  $filter . ')';
+
+        if ($this->_params['scope'] == 'one') {
+            $func = 'ldap_list';
+        } else {
+            $func = 'ldap_search';
+        }
+
+        $search = @$func($this->_ds, $this->_params['basedn'], $filter,
+                         array($this->_params['uid']));
         if (!$search) {
             Horde::logMessage(ldap_error($this->_ds), __FILE__, __LINE__, PEAR_LOG_ERR);
             return PEAR::raiseError(_("Could not search the LDAP server."));
@@ -163,7 +183,7 @@ class Auth_ldap extends Auth {
      *
      * @param string $dn     The dn of the user
      *
-     * @return array  array with keys being "shadowlastchange", "shadowmin"
+     * @return array  Array with keys being "shadowlastchange", "shadowmin"
      *                "shadowmax", "shadowwarning" and containing their
      *                respective values or false for no support.
      */
@@ -175,44 +195,69 @@ class Auth_ldap extends Auth {
                               'shadowmax' => false,
                               'shadowwarning' => false);
 
-
-        $result = @ldap_read($this->_ds, $dn, 'objectClass=*');
-
+        /* According to LDAP standard, to read operational attributes, you
+         * must request them explicitly. Attributes involved in password
+         * expiration policy:
+         *    pwdlastset: Active Directory
+         *    shadow*: shadowUser schema
+         *    passwordexpirationtime: Sun and Fedora Directory Server */
+        $result = @ldap_read($this->_ds, $dn, '(objectClass=*)',
+                             array('pwdlastset', 'shadowmax', 'shadowmin',
+                                   'shadowlastchange', 'shadowwarning',
+                                   'passwordexpirationtime'));
         if ($result) {
             $information = @ldap_get_entries($this->_ds, $result);
 
             if ($this->_params['ad']) {
                 if (isset($information[0]['pwdlastset'][0])) {
-                    /* Active Directory handles timestamps a bit
-                     * differently.  Convert the timestamp to a UNIX
-                     * timestamp. */
+                    /* Active Directory handles timestamps a bit differently.
+                     * Convert the timestamp to a UNIX timestamp. */
                     $lookupshadow['shadowlastchange'] = floor((($information[0]['pwdlastset'][0] / 10000000) - 11644406783) / 86400) - 1;
 
-                    /* Password expiry attributes are in a policy. We
-                     * cannot read them so use the Horde config. */
+                    /* Password expiry attributes are in a policy. We cannot
+                     * read them so use the Horde config. */
+                    $lookupshadow['shadowwarning'] = $this->_params['warnage'];
+                    $lookupshadow['shadowmin'] = $this->_params['minage'];
+                    $lookupshadow['shadowmax'] = $this->_params['maxage'];
+                }
+            } elseif (isset($information[0]['passwordexpirationtime'][0])) {
+                /* Sun/Fedora Directory Server uses a special attribute
+                 * passwordexpirationtime.  It has precedence over shadow*
+                 * because it actually locks the expired password at the LDAP
+                 * server level.  The correct way to check expiration should
+                 * be using LDAP controls, unfortunately PHP doesn't support
+                 * controls on bind() responses. */
+                $ldaptimepattern = "/([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})Z/";
+                if (preg_match($ldaptimepattern, $information[0]['passwordexpirationtime'][0], $regs)) {
+                    /* Sun/Fedora Directory Server return expiration time, not
+                     * last change time. We emulate the behaviour taking it
+                     * back to maxage. */
+                    $lookupshadow['shadowlastchange'] = floor(mktime($regs[4], $regs[5], $regs[6], $regs[2], $regs[3], $regs[1]) / 86400) - $this->_params['maxage'];
+
+                    /* Password expiry attributes are in not accessible policy
+                     * entry. */
                     $lookupshadow['shadowwarning'] = $this->_params['warnage'];
                     $lookupshadow['shadowmin']     = $this->_params['minage'];
                     $lookupshadow['shadowmax']     = $this->_params['maxage'];
+                } else {
+                    Horde::logMessage('Wrong time format: ' . $information[0]['passwordexpirationtime'][0], __FILE__, __LINE__, PEAR_LOG_ERR);
                 }
             } else {
                 if (isset($information[0]['shadowmax'][0])) {
                     $lookupshadow['shadowmax'] =
-                                             $information[0]['shadowmax'][0];
+                        $information[0]['shadowmax'][0];
                 }
-
                 if (isset($information[0]['shadowmin'][0])) {
                     $lookupshadow['shadowmin'] =
-                                             $information[0]['shadowmin'][0];
+                        $information[0]['shadowmin'][0];
                 }
-
                 if (isset($information[0]['shadowlastchange'][0])) {
                     $lookupshadow['shadowlastchange'] =
-                                      $information[0]['shadowlastchange'][0];
+                        $information[0]['shadowlastchange'][0];
                 }
-
                 if (isset($information[0]['shadowwarning'][0])) {
                     $lookupshadow['shadowwarning'] =
-                                         $information[0]['shadowwarning'][0];
+                        $information[0]['shadowwarning'][0];
                 }
             }
         }
@@ -233,9 +278,9 @@ class Auth_ldap extends Auth {
     function _authenticate($userId, $credentials)
     {
         /* Connect to the LDAP server. */
-        $success = $this->_connect();
-        if (is_a($success, 'PEAR_Error')) {
-            $this->_setAuthError(AUTH_REASON_MESSAGE, $success->getMessage());
+        $result = $this->_connect();
+        if (is_a($result, 'PEAR_Error')) {
+            $this->_setAuthError(AUTH_REASON_MESSAGE, $result->getMessage());
             return false;
         }
 
@@ -250,28 +295,21 @@ class Auth_ldap extends Auth {
         $bind = @ldap_bind($this->_ds, $dn, $credentials['password']);
         if ($bind != false) {
             if ($this->_params['password_expiration'] == 'yes') {
-
                 $shadow = $this->_lookupShadow($dn);
-
                 if ($shadow['shadowmax'] && $shadow['shadowlastchange'] &&
                     $shadow['shadowwarning']) {
-
                     $today = floor(time() / 86400);
                     $warnday = $shadow['shadowlastchange'] +
                                $shadow['shadowmax'] - $shadow['shadowwarning'];
-
                     $toexpire = $shadow['shadowlastchange'] +
                                 $shadow['shadowmax'] - $today;
 
-
                     if ($today >= $warnday) {
-                        $GLOBALS['notification']->push(sprintf(_("%d days until your password expires."), $toexpire),'horde.warning');
+                        $GLOBALS['notification']->push(sprintf(ngettext("%d day until your password expires.", "%d days until your password expires.", $toexpire), $toexpire), 'horde.warning');
                     }
-
-                    if ($toexpire = 0) {
+                    if ($toexpire == 0) {
                         $this->_authCredentials['changeRequested'] = true;
                     }
-
                     if ($toexpire < 0) {
                         $this->_setAuthError(AUTH_REASON_EXPIRED);
                         return false;
@@ -303,28 +341,32 @@ class Auth_ldap extends Auth {
         }
 
         /* Connect to the LDAP server. */
-        $success = $this->_connect();
+        $result = $this->_connect();
 
         global $conf;
         if (!empty($conf['hooks']['authldap'])) {
-            @include HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_horde_hook_authldap')) {
-                $entry = call_user_func('_horde_hook_authldap', $userId, $credentials);
-                $dn = $entry['dn'];
-                /* remove the dn entry from the array. */
-                unset($entry['dn']);
+            $entry = Horde::callHook('_horde_hook_authldap',
+                                     array($userId, $credentials));
+            if (is_a($entry, 'PEAR_Error')) {
+                return $entry;
             }
+            $dn = $entry['dn'];
+            /* Remove the dn entry from the array. */
+            unset($entry['dn']);
         } else {
             /* Try this simple default and hope it works. */
-            $dn = $this->_params['uid'] . '=' . $userId . ',' . $this->_params['basedn'];
+            $dn = $this->_params['uid'] . '=' . $userId . ','
+                . $this->_params['basedn'];
             $entry['cn'] = $userId;
             $entry['sn'] = $userId;
             $entry[$this->_params['uid']] = $userId;
-            $entry['objectclass'] = array_merge(array('top'),
-                                    $this->_params['newuser_objectclass']);
-            $entry['userPassword'] = $this->getCryptedPassword($credentials['password'],'',
-                                                               $this->_params['encryption'],
-                                                               'true');
+            $entry['objectclass'] = array_merge(
+                array('top'),
+                $this->_params['newuser_objectclass']);
+            $entry['userPassword'] = $this->getCryptedPassword(
+                $credentials['password'], '',
+                $this->_params['encryption'],
+                'true');
 
             if ($this->_params['password_expiration'] == 'yes') {
                 $entry['shadowMin'] = $this->_params['minage'];
@@ -333,10 +375,10 @@ class Auth_ldap extends Auth {
                 $entry['shadowLastChange'] = floor(time() / 86400);
             }
         }
-        $success = @ldap_add($this->_ds, $dn, $entry);
+        $result = @ldap_add($this->_ds, $dn, $entry);
 
-        if (!$success) {
-           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to add user %s. This is what the server said: "), $userId) . @ldap_error($this->_ds));
+        if (!$result) {
+           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to add user \"%s\". This is what the server said: "), $userId) . @ldap_error($this->_ds));
         }
 
         @ldap_close($this->_ds);
@@ -358,19 +400,18 @@ class Auth_ldap extends Auth {
         }
 
         /* Connect to the LDAP server. */
-        $success = $this->_connect();
-        if (is_a($success, 'PEAR_Error')) {
-            $this->_setAuthError(AUTH_REASON_MESSAGE, $success->getMessage());
+        $result = $this->_connect();
+        if (is_a($result, 'PEAR_Error')) {
+            $this->_setAuthError(AUTH_REASON_MESSAGE, $result->getMessage());
             return false;
         }
 
-        global $conf;
-        if (!empty($conf['hooks']['authldap'])) {
-            @include HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_horde_hook_authldap')) {
-                $entry = call_user_func('_horde_hook_authldap', $userId);
-                $dn = $entry['dn'];
+        if (!empty($GLOBALS['conf']['hooks']['authldap'])) {
+            $entry = Horde::callHook('_horde_hook_authldap', array($userId));
+            if (is_a($entry, 'PEAR_Error')) {
+                return $entry;
             }
+            $dn = $entry['dn'];
         } else {
             /* Search for the user's full DN. */
             $dn = $this->_findDN($userId);
@@ -380,9 +421,9 @@ class Auth_ldap extends Auth {
             }
         }
 
-        $success = @ldap_delete($this->_ds,$dn);
-        if (!$success) {
-           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to remove user %s"), $userId));
+        $result = @ldap_delete($this->_ds, $dn);
+        if (!$result) {
+           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to remove user \"%s\""), $userId));
         }
 
         @ldap_close($this->_ds);
@@ -406,30 +447,31 @@ class Auth_ldap extends Auth {
         }
 
         /* Connect to the LDAP server. */
-        $success = $this->_connect();
-        if (is_a($success, 'PEAR_Error')) {
-            return PEAR::raiseError($success->getMessage());
+        $result = $this->_connect();
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
         }
 
-        global $conf;
-        if (!empty($conf['hooks']['authldap'])) {
-            @include HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_horde_hook_authldap')) {
-                $entry = call_user_func('_horde_hook_authldap', $oldID, $credentials);
-                $olddn = $entry['dn'];
-                $entry = call_user_func('_horde_hook_authldap', $newID, $credentials);
-                $newdn = $entry['dn'];
-                unset($entry['dn']);
+        if (!empty($GLOBLS['conf']['hooks']['authldap'])) {
+            $entry = Horde::callHook('_horde_hook_authldap',
+                                     array($oldID, $credentials));
+            if (is_a($entry, 'PEAR_Error')) {
+                return $entry;
             }
+            $olddn = $entry['dn'];
+            $entry = Horde::callHook('_horde_hook_authldap',
+                                     array($newID, $credentials));
+            $newdn = $entry['dn'];
+            unset($entry['dn']);
         } else {
             /* Search for the user's full DN. */
             $dn = $this->_findDN($oldID);
 
             if (is_a($dn, 'PEAR_Error')) {
-                return PEAR::raiseError($success->getMessage());
+                return $dn;
             } else {
                 $olddn = $dn;
-                $newdn = preg_replace('/uid=.*?,/','uid='.$newID.',',$dn,1);
+                $newdn = preg_replace('/uid=.*?,/', 'uid=' . $newID . ',', $dn, 1);
                 $shadow = $this->_lookupShadow($dn);
 
                 /* if shadowmin hasn't yet expired only change when we are
@@ -446,23 +488,25 @@ class Auth_ldap extends Auth {
                 }
 
                 /* Encrypt the new password */
-                $entry['userpassword'] = $this->getCryptedPassword($credentials['password'],'',
-                                                                   $this->_params['encryption'],
-                                                                   'true');
+                $entry['userpassword'] = $this->getCryptedPassword(
+                    $credentials['password'], '',
+                    $this->_params['encryption'],
+                    'true');
             }
         }
 
         if ($oldID != $newID) {
             if (LDAP_OPT_PROTOCOL_VERSION == 3) {
-                ldap_rename($this->_ds, $olddn, $newdn, $this->_params['basedn'], true);
+                ldap_rename($this->_ds, $olddn, $newdn,
+                            $this->_params['basedn'], true);
 
-                $success = ldap_modify($this->_ds, $newdn, $entry);
+                $result = ldap_modify($this->_ds, $newdn, $entry);
             } else {
                 /* Get the complete old record first */
-                $success = @ldap_read($this->_ds, $olddn, 'objectClass=*');
+                $result = @ldap_read($this->_ds, $olddn, 'objectClass=*');
 
-                if ($success) {
-                    $information = @ldap_get_entries($this->_ds, $success);
+                if ($result) {
+                    $information = @ldap_get_entries($this->_ds, $result);
 
                     /* Remove the count elements from the array */
                     $counter = 0;
@@ -485,18 +529,18 @@ class Auth_ldap extends Auth {
                         $newrecord['shadowlastchange'] = $entry['shadowlastchange'];
                     }
 
-                    $success = ldap_add($this->_ds,$newdn,$newrecord);
-                    if ($success) {
-                        $success = @ldap_delete($this->_ds,$olddn);
+                    $result = ldap_add($this->_ds, $newdn, $newrecord);
+                    if ($result) {
+                        $result = @ldap_delete($this->_ds, $olddn);
                     }
                 }
             }
         } else {
-            $success = @ldap_modify($this->_ds, $olddn, $entry);
+            $result = @ldap_modify($this->_ds, $olddn, $entry);
         }
 
-        if (!$success) {
-            return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to update user %s"), $newID));
+        if (!$result) {
+            return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to update user \"%s\""), $newID));
         }
 
         @ldap_close($this->_ds);
@@ -512,24 +556,25 @@ class Auth_ldap extends Auth {
     function listUsers()
     {
         /* Connect to the LDAP server. */
-        $success = $this->_connect();
-        if (is_a($success, 'PEAR_Error')) {
-            return PEAR::raiseError($success->getMessage());
+        $result = $this->_connect();
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
         }
 
-        if (!empty($this->_params['filter'])) {
-            $filter = $this->_params['filter'];
-        } elseif (!is_array($this->_params['objectclass'])) {
-            $filter = 'objectclass=' . $this->_params['objectclass'];
+        $filter = $this->_getParamFilter();
+
+        if ($this->_params['scope'] == 'one') {
+            $func = 'ldap_list';
         } else {
-            $filter = '';
-            foreach ($this->_params['objectclass'] as $objectclass) {
-                $filter = '(&' . $filter;
-                $filter .= '(objectclass=' . $objectclass . '))';
-            }
+            $func = 'ldap_search';
         }
 
-        $search = @ldap_search($this->_ds, $this->_params['basedn'], $filter, array($this->_params['uid']));
+        /* Add a sizelimit, if specified. Default is 0, which means no limit.
+         * Note: You cannot override a server-side limit with this. */
+        $sizelimit = isset($this->_params['sizelimit']) ? $this->_params['sizelimit'] : 0;
+        $search = @$func($this->_ds, $this->_params['basedn'], $filter,
+                         array($this->_params['uid']), 0, $sizelimit);
+
         $entries = @ldap_get_entries($this->_ds, $search);
         $userlist = array();
         $uid = String::lower($this->_params['uid']);
@@ -540,4 +585,30 @@ class Auth_ldap extends Auth {
         return $userlist;
     }
 
+    /**
+     * Return a formatted LDAP filter as configured within the parameters.
+     *
+     * @return string LDAP search filter
+     * @access private
+     */
+    function _getParamFilter()
+    {
+        if (!empty($this->_params['filter'])) {
+            $filter = $this->_params['filter'];
+        } elseif (!is_array($this->_params['objectclass'])) {
+            $filter = 'objectclass=' . $this->_params['objectclass'];
+        } else {
+            $filter = '';
+            if (count($this->_params['objectclass']) > 1) {
+                $filter = '(&' . $filter;
+                foreach ($this->_params['objectclass'] as $objectclass) {
+                    $filter .= '(objectclass=' . $objectclass . ')';
+                }
+                $filter .= ')';
+            } elseif (count($this->_params['objectclass']) == 1) {
+                $filter = '(objectClass=' . $this->_params['objectclass'][0] . ')';
+            }
+        }
+        return $filter;
+    }
 }
