@@ -8,9 +8,9 @@ require_once 'Horde/Prefs.php';
  * methods take care of any site-specific restrictions configured in prefs.php
  * and conf.php.
  *
- * $Horde: framework/Prefs/Identity.php,v 1.1.2.12 2008/06/12 22:13:35 jan Exp $
+ * $Horde: framework/Prefs/Identity.php,v 1.1.2.14 2010/03/17 16:48:42 jan Exp $
  *
- * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2001-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -467,13 +467,22 @@ class Identity {
                 return array(_("Email addresses to confirm not found."), 'horde.message');
             } else {
                 $identity = $this->_prefs->convertFromDriver($confirm[$hash], NLS::getCharset());
-                $verified = array();
-                foreach ($identity as $key => $value) {
-                    if (!$this->_prefs->isLocked($key)) {
-                        $verified[$key] = $value;
+                $id = array_search($identity['id'], $this->getAll('id'));
+                if ($id === false) {
+                    /* Adding a new identity. */
+                    $verified = array();
+                    foreach ($identity as $key => $value) {
+                        if (!$this->_prefs->isLocked($key)) {
+                            $verified[$key] = $value;
+                        }
+                    }
+                    $this->add($verified);
+                } else {
+                    /* Updating an existing identity. */
+                    foreach ($identity as $key => $value) {
+                        $this->setValue($key, $value, $id);
                     }
                 }
-                $this->add($verified);
                 $this->save();
                 unset($confirm[$hash]);
                 $this->_prefs->setValue('confirm_email', serialize($confirm), false);
