@@ -6,7 +6,7 @@ require_once 'Horde/Util.php';
  * This class defines the Horde_Image:: API, and also provides some
  * utility functions, such as generating highlights of a color.
  *
- * $Horde: framework/Image/Image.php,v 1.39.10.20 2009/03/23 18:15:47 mrubinsk Exp $
+ * $Horde: framework/Image/Image.php,v 1.39.10.21 2010/03/28 16:39:11 mrubinsk Exp $
  *
  * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
@@ -54,13 +54,6 @@ class Horde_Image {
      */
     var $_data = '';
 
-    /**
-     * The current image id.
-     *
-     * @var string
-     */
-    var $_id = '';
-
     /**
      * The current width of the image data.
      *
@@ -311,7 +304,6 @@ class Horde_Image {
     function reset()
     {
         $this->_data = '';
-        $this->_id = '';
         $this->_width = null;
         $this->_height = null;
         $this->_background = 'white';
@@ -345,11 +337,8 @@ class Horde_Image {
      */
     function loadString($id, $image_data)
     {
-        if ($id != $this->_id) {
-            $this->reset();
-            $this->_data = $image_data;
-            $this->_id = $id;
-        }
+        $this->reset();
+        $this->_data = $image_data;
     }
 
     /**
@@ -364,16 +353,12 @@ class Horde_Image {
      */
     function loadFile($filename)
     {
-        if ($filename != $this->_id) {
-            $this->reset();
-            if (!file_exists($filename)) {
-                return PEAR::raiseError('The image file ' . $filename . ' does not exist.');
-            }
-            if ($this->_data = file_get_contents($filename)) {
-                $this->_id = $filename;
-            } else {
-                return PEAR::raiseError('Could not load the image file ' . $filename);
-            }
+        $this->reset();
+        if (!file_exists($filename)) {
+            return PEAR::raiseError('The image file ' . $filename . ' does not exist.');
+        }
+        if (!$this->_data = file_get_contents($filename)) {
+            return PEAR::raiseError('Could not load the image file ' . $filename);
         }
     }
 
