@@ -7,7 +7,7 @@ require_once dirname(__FILE__) . '/../Image.php';
  * extension. It mainly provides some utility functions, such as the
  * ability to make pixels, for now.
  *
- * $Horde: framework/Image/Image/gd.php,v 1.48.10.23 2009/04/14 04:13:20 mrubinsk Exp $
+ * $Horde: framework/Image/Image/gd.php,v 1.48.10.24 2010/03/28 16:39:11 mrubinsk Exp $
  *
  * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
@@ -43,13 +43,6 @@ class Horde_Image_gd extends Horde_Image {
      */
     var $_im;
 
-    /**
-     * String identifier of the current image. New image data will not be
-     * loaded if the same id is already loaded.
-     *
-     * @var string
-     */
-    var $_id = '';
 
     function Horde_Image_gd($params)
     {
@@ -185,18 +178,15 @@ class Horde_Image_gd extends Horde_Image {
      */
     function loadString($id, $image_data)
     {
-        if ($id != $this->_id) {
-            if ($this->_im) {
-                if (is_a($result = $this->reset(), 'PEAR_Error')) {
-                    return $result;
-                }
-            }
-            $this->_im = &$this->_call('imageCreateFromString', array($image_data));
-            $this->_id = $id;
-            if (is_a($this->_im, 'PEAR_Error')) {
-                return $this->_im;
+        if ($this->_im) {
+            if (is_a($result = $this->reset(), 'PEAR_Error')) {
+                return $result;
             }
         }
+        $this->_im = &$this->_call('imageCreateFromString', array($image_data));
+        if (is_a($this->_im, 'PEAR_Error')) {
+            return $this->_im;
+        }
     }
 
     /**
