@@ -2,7 +2,7 @@
 /**
  * Resource management for the Kolab server.
  *
- * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.6 2009/05/09 22:05:59 wrobel Exp $
+ * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.13 2010/04/19 11:57:21 wrobel Exp $
  *
  * PHP version 4
  *
@@ -24,6 +24,10 @@ require_once 'Horde/MIME/Headers.php';
 require_once 'Horde/MIME/Part.php';
 require_once 'Horde/MIME/Structure.php';
 
+/** Load Kolab_Filter elements */
+require_once 'Horde/Kolab/Resource/Epoch.php';
+require_once 'Horde/Kolab/Resource/Itip.php';
+
 require_once 'Horde/String.php';
 String::setDefaultCharset('utf-8');
 
@@ -42,7 +46,7 @@ define('RM_ITIP_TENTATIVE',                 3);
 /**
  * Provides Kolab resource handling
  *
- * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.6 2009/05/09 22:05:59 wrobel Exp $
+ * $Horde: framework/Kolab_Filter/lib/Horde/Kolab/Resource.php,v 1.15.2.13 2010/04/19 11:57:21 wrobel Exp $
  *
  * Copyright 2004-2009 Klarälvdalens Datakonsult AB
  *
@@ -105,7 +109,7 @@ class Kolab_Resource
         $policies = array();
         $defaultpolicy = false;
         foreach ($actions as $action) {
-            if (ereg('(.*):(.*)', $action, $regs)) {
+            if (preg_match('/(.*):(.*)/', $action, $regs)) {
                 $policies[strtolower($regs[1])] = $regs[2];
             } else {
                 $defaultpolicy = $action;
@@ -158,7 +162,7 @@ class Kolab_Resource
             if ($conttype == 'text/calendar') {
                 $part = $mime->getPart($mimeid);
 
-                $iCalendar = &new Horde_iCalendar();
+                $iCalendar = new Horde_iCalendar();
                 $iCalendar->parsevCalendar($part->transferDecode());
 
                 return $iCalendar;
@@ -173,7 +177,7 @@ class Kolab_Resource
         global $conf;
 
         // Handle virtual domains
-        list($user, $domain) = split('@', $id);
+        list($user, $domain) = explode('@', $id);
         if (empty($domain)) {
             $domain = $conf['kolab']['filter']['email_domain'];
         }
@@ -211,8 +215,8 @@ class Kolab_Resource
         require_once 'Horde/Kolab/Storage/List.php';
         $list = &Kolab_List::singleton();
         $default = $list->getForeignDefault($id, 'event');
-        if (!$default) {
-            $default = &new Kolab_Folder();
+        if (!$default || is_a($default, 'PEAR_Error')) {
+            $default = new Kolab_Folder();
             $default->setList($list);
             $default->setName($conf['kolab']['filter']['calendar_store']);
             //FIXME: The calendar user needs access here
@@ -228,102 +232,6 @@ class Kolab_Resource
         return $default;
     }
 
-    function _objectFromItip(&$itip)
-    {
-        $object = array();
-        $object['uid'] = $itip->getAttributeDefault('UID', '');
-
-        $org_params = $itip->getAttribute('ORGANIZER', true);
-        if (!is_a( $org_params, 'PEAR_Error')) {
-            if (!empty($org_params[0]['CN'])) {
-                $object['organizer']['display-name'] = $org_params[0]['CN'];
-            }
-            $orgemail = $itip->getAttributeDefault('ORGANIZER', '');
-            if (eregi('mailto:(.*)', $orgemail, $regs )) {
-                $orgemail = $regs[1];
-            }
-            $object['organizer']['smtp-address'] = $orgemail;
-        }
-        $object['summary'] = $itip->getAttributeDefault('SUMMARY', '');
-        $object['location'] = $itip->getAttributeDefault('LOCATION', '');
-        $object['body'] = $itip->getAttributeDefault('DESCRIPTION', '');
-        $dtend = $itip->getAttributeDefault('DTEND', '');
-        if (is_array($dtend)) {
-            $object['_is_all_day'] = true;
-        }
-        $object['start-date'] = $this->convert2epoch($itip->getAttributeDefault('DTSTART', ''));
-        $object['end-date'] = $this->convert2epoch($dtend);
-
-        $attendees = $itip->getAttribute('ATTENDEE');
-        if (!is_a( $attendees, 'PEAR_Error')) {
-            $attendees_params = $itip->getAttribute('ATTENDEE', true);
-            if (!is_array($attendees)) {
-                $attendees = array($attendees);
-            }
-            if (!is_array($attendees_params)) {
-                $attendees_params = array($attendees_params);
-            }
-
-            $object['attendee'] = array();
-            for ($i = 0; $i < count($attendees); $i++) {
-                $attendee = array();
-                if (isset($attendees_params[$i]['CN'])) {
-                    $attendee['display-name'] = $attendees_params[$i]['CN'];
-                }
-
-                $attendeeemail = $attendees[$i];
-                if (eregi('mailto:(.*)', $attendeeemail, $regs)) {
-                    $attendeeemail = $regs[1];
-                }
-                $attendee['smtp-address'] = $attendeeemail;
-
-                if( $attendees_params[$i]['RSVP'] == 'FALSE' ) {
-                    $attendee['request-response'] = false;
-                } else {
-                    $attendee['request-response'] = true;
-                }
-
-                if (isset($attendees_params[$i]['ROLE'])) {
-                    $attendee['role'] = $attendees_params[$i]['ROLE'];
-                }
-                $object['attendee'][] = $attendee;
-            }
-        }
-
-        // Alarm
-        $valarm = $itip->findComponent('VALARM');
-        if ($valarm) {
-            $trigger = $valarm->getAttribute('TRIGGER');
-            if (!is_a($trigger, 'PEAR_Error')) {
-                $p = $valarm->getAttribute('TRIGGER', true);
-                if ($trigger < 0) {
-                    // All OK, enter the alarm into the XML
-                    // NOTE: The Kolab XML format seems underspecified
-                    // wrt. alarms currently...
-                    $object['alarm'] = -$trigger / 60;
-                }
-            } else {
-                Horde::logMessage('No TRIGGER in VALARM. ' . $trigger->getMessage(),
-                                  __FILE__, __LINE__, PEAR_LOG_ERR);
-            }
-        }
-
-        // Recurrence
-        $rrule_str = $itip->getAttribute('RRULE');
-        if (!is_a($rrule_str, 'PEAR_Error')) {
-            require_once 'Horde/Date/Recurrence.php';
-            $recurrence = &new Horde_Date_Recurrence(time());
-            $recurrence->fromRRule20($rrule_str);
-            $object['recurrence'] = $recurrence->toHash();
-        }
-
-        Horde::logMessage(sprintf('Assembled event object: %s',
-                                  print_r($object, true)),
-                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
-
-        return $object;
-    }
-
     function handleMessage($fqhostname, $sender, $resource, $tmpfname)
     {
         global $conf;
@@ -373,31 +281,38 @@ class Kolab_Resource
                               __FILE__, __LINE__, PEAR_LOG_INFO);
             return true;
         }
+        $itip = new Horde_Kolab_Resource_Itip($itip);
 
         // What is the request's method? i.e. should we create a new event/cancel an
         // existing event, etc.
-        $method = strtoupper($iCalendar->getAttributeDefault('METHOD',
-                                                             $itip->getAttributeDefault('METHOD', 'REQUEST')));
+        $method = strtoupper(
+            $iCalendar->getAttributeDefault(
+                'METHOD',
+                $itip->getMethod()
+            )
+        );
 
         // What resource are we managing?
         Horde::logMessage(sprintf('Processing %s method for %s', $method, $id),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         // This is assumed to be constant across event creation/modification/deletipn
-        $uid = $itip->getAttributeDefault('UID', '');
+        $uid = $itip->getUid();
         Horde::logMessage(sprintf('Event has UID %s', $uid),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         // Who is the organiser?
-        $organiser = preg_replace('/^mailto:\s*/i', '', $itip->getAttributeDefault('ORGANIZER', ''));
+        $organiser = $itip->getOrganizer();
         Horde::logMessage(sprintf('Request made by %s', $organiser),
                       __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         // What is the events summary?
-        $summary = $itip->getAttributeDefault('SUMMARY', '');
+        $summary = $itip->getSummary();
 
-        $dtstart = $this->convert2epoch($itip->getAttributeDefault('DTSTART', 0));
-        $dtend = $this->convert2epoch($itip->getAttributeDefault('DTEND', 0));
+        $estart = new Horde_Kolab_Resource_Epoch($itip->getStart());
+        $dtstart = $estart->getEpoch();
+        $eend = new Horde_Kolab_Resource_Epoch($itip->getEnd());
+        $dtend = $eend->getEpoch();
 
         Horde::logMessage(sprintf('Event starts on <%s> %s and ends on <%s> %s.',
                                   $dtstart, $this->iCalDate2Kolab($dtstart), $dtend, $this->iCalDate2Kolab($dtend)),
@@ -463,7 +378,7 @@ class Kolab_Resource
             }
 
             /** Generate the Kolab object */
-            $object = $this->_objectFromItip($itip);
+            $object = $itip->getKolabObject();
 
             $outofperiod=0;
 
@@ -491,7 +406,8 @@ class Kolab_Resource
                                           $vfbstart, $this->iCalDate2Kolab($vfbstart), $vfbend, $this->iCalDate2Kolab($vfbend)),
                                   __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-                if ($vfbstart && $dtstart > $this->convert2epoch ($vfbend)) {
+                $evfbend = new Horde_Kolab_Resource_Epoch($vfbend);
+                if ($vfbstart && $dtstart > $evfbend->getEpoch()) {
                     $outofperiod=1;
                 } else {
                     // Check whether we are busy or not
@@ -505,23 +421,26 @@ class Kolab_Resource
                                       __FILE__, __LINE__, PEAR_LOG_DEBUG);
                     $conflict = false;
                     if (!empty($object['recurrence'])) {
-                        $recurrence = &new Horde_Date_Recurrence(time());
+                        $recurrence = new Horde_Date_Recurrence(time());
                         $recurrence->fromHash($object['recurrence']);
                         $duration = $dtend - $dtstart;
                         $events = array();
                         $next_start = $vfbstart;
-                        while ($next = $recurrence->nextActiveRecurrence($next_start)) {
-                            $next_start = $next->timestamp();
-                            if ($next_start < $vfbend) {
-                                $events[$next_start] = $next_start + $duration;
-                            } else {
-                                break;
-                            }
+                        $next = $recurrence->nextActiveRecurrence($vfbstart);
+                        while ($next !== false && $next->compareDate($vfbend) <= 0) {
+                            $next_ts = $next->timestamp();
+                            $events[$next_ts] = $next_ts + $duration;
+                            $next = $recurrence->nextActiveRecurrence(array('year' => $next->year,
+                                                                            'month' => $next->month,
+                                                                            'mday' => $next->mday + 1,
+                                                                            'hour' => $next->hour,
+                                                                            'min' => $next->min,
+                                                                            'sec' => $next->sec));
                         }
                     } else {
                         $events = array($dtstart => $dtend);
                     }
-                    
+
                     foreach ($events as $dtstart => $dtend) {
                         foreach ($busyperiods as $busyfrom => $busyto) {
                             if (empty($busyfrom) && empty($busyto)) {
@@ -624,39 +543,14 @@ class Kolab_Resource
                 }
             }
 
-            $result = $data->save($object, $old_uid);
+            $itip->setAccepted($resource);
+
+            $result = $data->save($itip->getKolabObject(), $old_uid);
             if (is_a($result, 'PEAR_Error')) {
                 $result->code = OUT_LOG | EX_UNAVAILABLE;
                 return $result;
             }
 
-            // Update our status within the iTip request and send the reply
-            $itip->setAttribute('STATUS', 'CONFIRMED', array(), false);
-            $attendees = $itip->getAttribute('ATTENDEE');
-            if (!is_array($attendees)) {
-                $attendees = array($attendees);
-            }
-            $attparams = $itip->getAttribute('ATTENDEE', true);
-            foreach ($attendees as $i => $attendee) {
-                $attendee = preg_replace('/^mailto:\s*/i', '', $attendee);
-                if ($attendee != $resource) {
-                    continue;
-                }
-
-                $attparams[$i]['PARTSTAT'] = 'ACCEPTED';
-                if (array_key_exists('RSVP', $attparams[$i])) {
-                    unset($attparams[$i]['RSVP']);
-                }
-            }
-
-            // Re-add all the attendees to the event, using our updates status info
-            $firstatt = array_pop($attendees);
-            $firstattparams = array_pop($attparams);
-            $itip->setAttribute('ATTENDEE', $firstatt, $firstattparams, false);
-            foreach ($attendees as $i => $attendee) {
-                $itip->setAttribute('ATTENDEE', $attendee, $attparams[$i]);
-            }
-
             if ($outofperiod) {
                 $this->sendITipReply($cn, $resource, $itip, RM_ITIP_TENTATIVE,
                                      $organiser, $uid, $is_update);
@@ -672,11 +566,6 @@ class Kolab_Resource
             Horde::logMessage(sprintf('Removing event %s', $uid),
                               __FILE__, __LINE__, PEAR_LOG_INFO);
 
-            /* Ensure that $summary is single byte here, else we get
-             * problems with sprintf()
-             */
-            $summary = String::convertCharset($summary, 'utf-8', 'iso-8859-1');
-
             if (is_a($imap_error, 'PEAR_Error')) {
                 $body = sprintf(_("Unable to access %s's calendar:"), $resource) . "\n\n" . $summary;
                 $subject = sprintf(_("Error processing \"%s\""), $summary);
@@ -706,22 +595,16 @@ class Kolab_Resource
                 $subject = sprintf(_("%s has been cancelled"), $summary);
             }
 
-            /* Switch back to utf-8 */
-            $body = String::convertCharset($body, 'iso-8859-1');
-            $subject = String::convertCharset($subject, 'iso-8859-1');
-
             Horde::logMessage(sprintf('Sending confirmation of cancelation to %s', $organiser),
                               __FILE__, __LINE__, PEAR_LOG_WARNING);
 
-            $body = &new MIME_Part('text/plain', String::wrap($body, 76, "\n", 'utf-8'), 'utf-8');
+            $body = new MIME_Part('text/plain', String::wrap($body, 76, "\n", 'utf-8'), 'utf-8');
             $mime = &MIME_Message::convertMimePart($body);
             $mime->setTransferEncoding('quoted-printable');
             $mime->transferEncodeContents();
 
             // Build the reply headers.
-            $msg_headers = &new MIME_Headers();
-            $msg_headers->addReceivedHeader();
-            $msg_headers->addMessageIdHeader();
+            $msg_headers = new MIME_Headers();
             $msg_headers->addHeader('Date', date('r'));
             $msg_headers->addHeader('From', $resource);
             $msg_headers->addHeader('To', $organiser);
@@ -854,7 +737,7 @@ class Kolab_Resource
 
         $parsed = parse_url($fb_url);
 
-        list($user, $domain) = split('@', $email);
+        list($user, $domain) = explode('@', $email);
         if (empty($domain)) {
             $domain = $conf['kolab']['filter']['email_domain'];
         }
@@ -874,7 +757,7 @@ class Kolab_Resource
         @$http->sendRequest();
         if ($http->getResponseCode() != 200) {
             return PEAR::raiseError(sprintf('Unable to retrieve free/busy information for %s',
-                                            $email), 
+                                            $email),
                                     OUT_LOG | EX_UNAVAILABLE);
         }
         $vfb_text = $http->getResponseBody();
@@ -914,7 +797,7 @@ class Kolab_Resource
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         // Build the reply.
-        $vCal = &new Horde_iCalendar();
+        $vCal = new Horde_iCalendar();
         $vCal->setAttribute('PRODID', '-//kolab.org//NONSGML Kolab Server 2//EN');
         $vCal->setAttribute('METHOD', 'REPLY');
 
@@ -955,7 +838,7 @@ class Kolab_Resource
         //     for( $i = 0; $i < count($attendees); $i++ ) {
         //       $attendee = preg_replace('/^mailto:\s*/i', '', $attendees[$i]);
         //       if ($attendee != $resource) {
-        // 	continue;
+        //	continue;
         //       }
         //       $params = $params[$i];
         //       break;
@@ -1003,7 +886,7 @@ class Kolab_Resource
         $itip_reply->setAttribute('ATTENDEE', 'MAILTO:' . $resource, $params);
         $vCal->addComponent($itip_reply);
 
-        $ics = &new MIME_Part('text/calendar', $vCal->exportvCalendar(), 'UTF-8' );
+        $ics = new MIME_Part('text/calendar', $vCal->exportvCalendar(), 'UTF-8' );
         //$ics->setName('event-reply.ics');
         $ics->setContentTypeParameter('method', 'REPLY');
 
@@ -1021,9 +904,7 @@ class Kolab_Resource
         $mime->transferEncodeContents();
 
         // Build the reply headers.
-        $msg_headers = &new MIME_Headers();
-        $msg_headers->addReceivedHeader();
-        $msg_headers->addMessageIdHeader();
+        $msg_headers = new MIME_Headers();
         $msg_headers->addHeader('Date', date('r'));
         $msg_headers->addHeader('From', "$cn <$resource>");
         $msg_headers->addHeader('To', $organiser);
@@ -1062,31 +943,6 @@ class Kolab_Resource
         }
     }
 
-
-    /**
-     * Clear information from a date array.
-     *
-     * @param array $ical_date  The array to clear.
-     *
-     * @return array The cleaned array.
-     */
-    function cleanArray($ical_date)
-    {
-        if (array_key_exists('hour', $ical_date)) {
-            $temp['hour']   = array_key_exists('hour', $ical_date) ? $ical_date['hour'] :  '00';
-            $temp['minute']   = array_key_exists('minute', $ical_date) ? $ical_date['minute'] :  '00';
-            $temp['second']   = array_key_exists('second', $ical_date) ? $ical_date['second'] :  '00';
-            $temp['zone']   = array_key_exists('zone', $ical_date) ? $ical_date['zone'] :  'UTC';
-        } else {
-            $temp['DATE'] = '1';
-        }
-        $temp['year']   = array_key_exists('year', $ical_date) ? $ical_date['year'] :  '0000';
-        $temp['month']   = array_key_exists('month', $ical_date) ? $ical_date['month'] :  '00';
-        $temp['mday']   = array_key_exists('mday', $ical_date) ? $ical_date['mday'] :  '00';
-
-        return $temp;
-    }
-
     /**
      * Conveert iCal dates to Kolab format.
      *
@@ -1111,8 +967,9 @@ class Kolab_Resource
             $temp = $this->cleanArray($ical_date);
             if (array_key_exists('DATE', $temp)) {
                 if ($type == 'ENDDATE') {
+                    $etemp = new Horde_Kolab_Resource_Epoch($temp);
                     // substract a day (86400 seconds) using epochs to take number of days per month into account
-                    $epoch= $this->convert2epoch($temp) - 86400;
+                    $epoch= $etemp->getEpoch() - 86400;
                     $date = gmstrftime('%Y-%m-%d', $epoch);
                 } else {
                     $date= sprintf('%04d-%02d-%02d', $temp['year'], $temp['month'], $temp['mday']);
@@ -1131,30 +988,4 @@ class Kolab_Resource
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
         return $date;
     }
-
-    /**
-     * Convert a date to an epoch.
-     *
-     * @param array  $values  The array to convert.
-     *
-     * @return int Time.
-     */
-    function convert2epoch($values)
-    {
-        Horde::logMessage(sprintf('Converting to epoch %s',
-                                  print_r($values, true)),
-                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
-
-        if (is_array($values)) {
-            $temp = $this->cleanArray($values);
-            $epoch = gmmktime($temp['hour'], $temp['minute'], $temp['second'],
-                              $temp['month'], $temp['mday'], $temp['year']);
-        } else {
-            $epoch=$values;
-        }
-
-        Horde::logMessage(sprintf('Converted <%s>', $epoch),
-                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        return $epoch;
-    }
 }
