@@ -3,15 +3,14 @@
  * The MIME_Viewer_report class is a wrapper used to load the appropriate
  * MIME_Viewer for multipart/report data (RFC 3462).
  *
- * $Horde: framework/MIME/MIME/Viewer/report.php,v 1.6.10.7 2007/01/02 13:54:26 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/report.php,v 1.6.10.11 2009/01/06 15:23:21 jan Exp $
  *
- * Copyright 2003-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2003-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@horde.org>
  * @package Horde_MIME_Viewer
  */
 class MIME_Viewer_report extends MIME_Viewer {
@@ -67,17 +66,22 @@ class MIME_Viewer_report extends MIME_Viewer {
      */
     function _resolveViewer()
     {
-        $viewer = null;
+        $type = $viewer = null;
 
         if (empty($this->_viewer)) {
-            if (!($type = $this->mime_part->getContentTypeParameter('report-type'))) {
-                return false;
+            if (($type = $this->mime_part->getContentTypeParameter('report-type'))) {
+                $viewer = &MIME_Viewer::factory($this->mime_part, 'message/' . String::lower($type));
+                $type = $this->mime_part->getPrimaryType();
+            } else {
+                /* If report-type is missing, the message is an improper
+                 * multipart/report message.  Attempt to fall back to a
+                 * multipart/mixed viewer instead. */
+                $type = 'multipart';
             }
 
-            $viewer = &MIME_Viewer::factory($this->mime_part, 'message/' . String::lower($type));
             if (empty($viewer) ||
                 (String::lower(get_class($viewer)) == 'mime_viewer_default')) {
-                if (!($viewer = &MIME_Viewer::factory($this->mime_part, $this->mime_part->getPrimaryType() . '/*'))) {
+                if (!($viewer = &MIME_Viewer::factory($this->mime_part, $type . '/*'))) {
                     return false;
                 }
             }
