@@ -1,15 +1,15 @@
 <?php
 
 require_once 'Horde/DataTree.php';
+require_once 'Horde/Cache.php';
 
 /**
  * The Perms_datatree:: class provides a DataTree driver for the Horde
  * permissions system.
  *
- * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.13 2007/01/02 13:54:34 jan Exp $
+ * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.24 2009/08/14 21:45:52 jan Exp $
  *
- * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2004-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -29,6 +29,13 @@ class Perms_datatree extends Perms {
      */
     var $_datatree;
 
+    /**
+     * Pointer to a Horde_Cache instance
+     *
+     * @var Horde_Cache
+     */
+    var $_cache;
+
     /**
      * Constructor.
      */
@@ -44,37 +51,9 @@ class Perms_datatree extends Perms {
         $this->_datatree = &DataTree::singleton($driver,
                                                 array_merge(Horde::getDriverConfig('datatree', $driver),
                                                             array('group' => 'horde.perms')));
-    }
 
-    /**
-     * Returns the available permissions for a given level.
-     *
-     * @param string $name  The permission's name.
-     *
-     * @return array  An array of available permissions and their titles.
-     */
-    function getAvailable($name)
-    {
-        if ($name == DATATREE_ROOT) {
-            $name = '';
-        }
-        return parent::getAvailable($name);
-    }
-
-    /**
-     * Given a permission name, returns the title for that permission by
-     * looking it up in the applications's permission api.
-     *
-     * @param string $name  The permissions's name.
-     *
-     * @return string  The title for the permission.
-     */
-    function getTitle($name)
-    {
-        if ($name == DATATREE_ROOT) {
-            return _("All Permissions");
-        }
-        return parent::getTitle($name);
+        $this->_cache = Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'],
+                                               Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
     }
 
     /**
@@ -86,10 +65,6 @@ class Perms_datatree extends Perms {
      */
     function &newPermission($name)
     {
-        if (empty($name)) {
-            return PEAR::raiseError('Permission names must be non-empty');
-        }
-
         $type = 'matrix';
         $params = null;
         if ($pos = strpos($name, ':')) {
@@ -103,6 +78,7 @@ class Perms_datatree extends Perms {
                 }
             }
         }
+
         $perm = &new DataTreeObject_Permission($name, $type, $params);
         $perm->setDataTree($this->_datatree);
 
@@ -125,7 +101,15 @@ class Perms_datatree extends Perms {
             return $permsCache[$name];
         }
 
-        $permsCache[$name] = $this->_datatree->getObject($name, 'DataTreeObject_Permission');
+        $perm = $this->_cache->get('perm_' . $name, $GLOBALS['conf']['cache']['default_lifetime']);
+        if ($perm === false) {
+            $perm = $this->_datatree->getObject($name, 'DataTreeObject_Permission');
+            $this->_cache->set('perm_' . $name, serialize($perm), $GLOBALS['conf']['cache']['default_lifetime']);
+            $permsCache[$name] = $perm;
+        } else {
+            $permsCache[$name] = unserialize($perm);
+        }
+
         return $permsCache[$name];
     }
 
@@ -137,8 +121,8 @@ class Perms_datatree extends Perms {
      */
     function &getPermissionById($cid)
     {
-        if ($cid == DATATREE_ROOT) {
-            $object = &$this->newPermission(DATATREE_ROOT);
+        if ($cid == PERMS_ROOT) {
+            $object = &$this->newPermission(PERMS_ROOT);
         } else {
             $object = &$this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
         }
@@ -158,6 +142,13 @@ class Perms_datatree extends Perms {
             return PEAR::raiseError('Permissions must be DataTreeObject_Permission objects or extend that class.');
         }
 
+        $name = $perm->getName();
+        if (empty($name)) {
+            return PEAR::raiseError('Permission names must be non-empty');
+        }
+        $this->_cache->expire('perm_' . $name);
+        $this->_cache->expire('perm_exists_' . $name);
+
         return $this->_datatree->add($perm);
     }
 
@@ -173,93 +164,13 @@ class Perms_datatree extends Perms {
             return PEAR::raiseError('Permissions must be DataTreeObject_Permission objects or extend that class.');
         }
 
-        return $this->_datatree->remove($perm->getName(), $force);
-    }
-
-    /**
-     * Finds out what rights the given user has to this object.
-     *
-     * @param mixed $permission  The full permission name of the object to
-     *                           check the permissions of, or the
-     *                           DataTreeObject_Permission object.
-     * @param string $user       The user to check for. Defaults to the current
-     *                           user.
-     * @param string $creator    The user who created the object.
-     *
-     * @return mixed  A bitmask of permissions, a permission value, or an array
-     *                of permission values the user has, depending on the
-     *                permission type and whether the permission value is
-     *                ambiguous. False if there is no such permsission.
-     */
-    function getPermissions($permission, $user = null, $creator = null)
-    {
-        if (!is_a($permission, 'DataTreeObject_Permission')) {
-            $permission = &$this->getPermission($permission);
-            if (is_a($permission, 'PEAR_Error')) {
-                Horde::logMessage($permission, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                return false;
-            }
+        $keys = $this->_datatree->get(DATATREE_FORMAT_FLAT, $perm->name, true);
+        foreach ($keys as $key) {
+            $this->_cache->expire('perm_' . $key);
+            $this->_cache->expire('perm_exists_' . $key);
         }
 
-        if ($user === null) {
-            $user = Auth::getAuth();
-        }
-
-        // If this is a guest user, only check guest permissions.
-        if (empty($user)) {
-            return $permission->getGuestPermissions();
-        }
-
-        // If $creator was specified, check creator permissions.
-        if ($creator !== null) {
-            // If the user is the creator of the event see if there
-            // are creator permissions.
-            if (strlen($user) && $user === $creator &&
-                ($perms = $permission->getCreatorPermissions()) !== null) {
-                return $perms;
-            }
-        }
-
-        // Check user-level permissions.
-        $userperms = $permission->getUserPermissions();
-        if (isset($userperms[$user])) {
-            return $userperms[$user];
-        }
-
-        // If no user permissions are found, try group permissions.
-        if (isset($permission->data['groups']) &&
-            is_array($permission->data['groups']) &&
-            count($permission->data['groups'])) {
-            require_once 'Horde/Group.php';
-            $groups = &Group::singleton();
-
-            $composite_perm = null;
-            $type = $permission->get('type');
-            foreach ($permission->data['groups'] as $group => $perm) {
-                if ($groups->userIsInGroup($user, $group)) {
-                    if ($composite_perm === null) {
-                        $composite_perm = $type == 'matrix' ? 0 : array();
-                    }
-                    if ($type == 'matrix') {
-                        $composite_perm |= $perm;
-                    } else {
-                        $composite_perm[] = $perm;
-                    }
-                }
-            }
-
-            if ($composite_perm !== null) {
-                return $composite_perm;
-            }
-        }
-
-        // If there are default permissions, return them.
-        if (($perms = $permission->getDefaultPermissions()) !== null) {
-            return $perms;
-        }
-
-        // Otherwise, deny all permissions to the object.
-        return false;
+        return $this->_datatree->remove($perm->name, $force);
     }
 
     /**
@@ -275,22 +186,6 @@ class Perms_datatree extends Perms {
         return $this->_datatree->getId($permission->getName());
     }
 
-    /**
-     * Finds out if the user has the specified rights to the given object.
-     *
-     * @param string $permission  The permission to check.
-     * @param string $user        The user to check for.
-     * @param integer $perm       The permission level that needs to be checked
-     *                            for.
-     * @param string $creator     The creator of the event
-     *
-     * @return boolean  True if the user has the specified permissions.
-     */
-    function hasPermission($permission, $user, $perm, $creator = null)
-    {
-        return ($this->getPermissions($permission, $user, $creator) & $perm);
-    }
-
     /**
      * Checks if a permission exists in the system.
      *
@@ -300,7 +195,14 @@ class Perms_datatree extends Perms {
      */
     function exists($permission)
     {
-        return $this->_datatree->exists($permission);
+        $key = 'perm_exists_' . $permission;
+        $exists = $this->_cache->get($key, $GLOBALS['conf']['cache']['default_lifetime']);
+        if ($exists === false) {
+            $exists = $this->_datatree->exists($permission);
+            $this->_cache->set($key, (string)$exists);
+        }
+
+        return (bool)$exists;
     }
 
     /**
@@ -315,6 +217,20 @@ class Perms_datatree extends Perms {
         return $this->_datatree->getParents($child);
     }
 
+    /**
+     * Returns a child's direct parent ID.
+     *
+     * @param mixed $child  Either the object, an array containing the
+     *                      path elements, or the object name for which
+     *                      to look up the parent's ID.
+     *
+     * @return mixed  The unique ID of the parent or PEAR_Error on error.
+     */
+    function getParent($child)
+    {
+        return $this->_datatree->getParent($child);
+    }
+
     /**
      * Returns all permissions of the system in a tree format.
      *
@@ -322,7 +238,7 @@ class Perms_datatree extends Perms {
      */
     function &getTree()
     {
-        $tree = $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
+        $tree = $this->_datatree->get(DATATREE_FORMAT_FLAT, PERMS_ROOT, true);
         return $tree;
     }
 
@@ -334,6 +250,10 @@ class Perms_datatree extends Perms {
  * information, you should extend this class instead of extending
  * DataTreeObject directly.
  *
+ * @TODO This class duplicates most of the functionality of the
+ * Horde_Permission class. However, because for BC/DataTree reasons it
+ * must extend DataTreeObject, we can't remove these methods yet.
+ *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
  * @since   Horde 2.1
@@ -369,7 +289,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     function get($attribute)
     {
         $value = parent::get($attribute);
-        if ($value === null && $attribute == 'type') {
+        if (is_null($value) && $attribute == 'type') {
             $value = 'matrix';
         }
         return $value;
@@ -428,7 +348,7 @@ class DataTreeObject_Permission extends DataTreeObject {
                 }
 
                 /* If nothing to actually update finish with this case. */
-                if ($perm_values === null) {
+                if (is_null($perm_values)) {
                     continue;
                 }
 
@@ -872,4 +792,23 @@ class DataTreeObject_Permission extends DataTreeObject {
         }
     }
 
+    /**
+     * Saves any changes to this object to the backend permanently. New objects
+     * are added instead.
+     *
+     * @return boolean|PEAR_Error  PEAR_Error on failure.
+     */
+    function save()
+    {
+        $name = $this->getName();
+        if (empty($name)) {
+            return PEAR::raiseError('Permission names must be non-empty');
+        }
+        parent::save();
+
+        $cache = Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
+        $cache->expire('perm_' . $name);
+        $cache->expire('perm_exists_' . $name);
+    }
+
 }
