@@ -7,9 +7,9 @@
  *   'timeout'    The period (in seconds) after which an id is purged.
  *                Defaults to 86400 (i.e. 24 hours).</pre>
  *
- * $Horde: framework/Token/Token/file.php,v 1.19.6.8 2007/01/02 13:54:45 jan Exp $
+ * $Horde: framework/Token/Token/file.php,v 1.19.6.11 2009/01/06 15:23:44 jan Exp $
  *
- * Copyright 1999-2007 Max Kalika <max@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -92,7 +92,7 @@ class Horde_Token_file extends Horde_Token {
         }
 
         /* Find already used IDs. */
-        $fileContents = @file($this->_params['token_dir'] . '/conn_' . $this->encodeRemoteAddress());
+        $fileContents = file($this->_params['token_dir'] . '/conn_' . $this->encodeRemoteAddress());
         if ($fileContents) {
             $iMax = count($fileContents);
             for ($i = 0; $i < $iMax; $i++) {
