@@ -13,9 +13,9 @@ define('HORDE_ERROR_DRIVER_CONFIG', 2);
 /**
  * The Util:: class provides generally useful methods of different kinds.
  *
- * $Horde: framework/Util/Util.php,v 1.384.6.33 2008/10/30 14:35:03 chuck Exp $
+ * $Horde: framework/Util/Util.php,v 1.384.6.37 2009/07/21 18:17:23 slusarz Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -53,12 +53,7 @@ class Util {
             return $ret;
         }
 
-        if (version_compare(zend_version(), '2', '>')) {
-            $ret = clone($obj);
-        } else {
-            $ret = unserialize(serialize($obj));
-        }
-
+        $ret = unserialize(serialize($obj));
         return $ret;
     }
 
@@ -646,7 +641,7 @@ class Util {
      * iterate through, deleting folders recursively.
      *
      * Contains code from gpg_functions.php.
-     * Copyright (c) 2002-2003 Braverock Ventures
+     * Copyright 2002-2003 Braverock Ventures
      *
      * @access private
      */
@@ -862,11 +857,12 @@ class Util {
      */
     function getPathInfo()
     {
-        if (isset($_SERVER['PATH_INFO'])) {
+        if (isset($_SERVER['PATH_INFO']) &&
+            strpos($_SERVER['SERVER_SOFTWARE'], 'lighttpd') === false) {
             return $_SERVER['PATH_INFO'];
         } elseif (isset($_SERVER['REQUEST_URI']) &&
                   isset($_SERVER['SCRIPT_NAME'])) {
-            $search = array($_SERVER['SCRIPT_NAME']);
+            $search = array((basename($_SERVER['SCRIPT_NAME']) == 'index.php') ? dirname($_SERVER['SCRIPT_NAME']) . '/' : $_SERVER['SCRIPT_NAME']);
             $replace = array('');
             if (!empty($_SERVER['QUERY_STRING'])) {
                 $search[] = '?' . $_SERVER['QUERY_STRING'];
