@@ -1,18 +1,41 @@
 <?php
 /**
- * $Horde: horde/login.php,v 2.175.2.4 2005/08/16 13:07:46 jan Exp $
+ * $Horde: horde/login.php,v 2.175.2.17 2009/01/06 15:13:50 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
+/* Add anchor to outgoing URL. */
+function _addAnchor($url, $type)
+{
+    switch ($type) {
+    case 'param':
+        if (!empty($GLOBALS['url_anchor'])) {
+            $url .= '#' . $GLOBALS['url_anchor'];
+        }
+        break;
+
+    case 'url':
+        $anchor = Util::getFormData('anchor_string');
+        if (!empty($anchor)) {
+            $url .= '#' . $anchor;
+        } else {
+            return _addAnchor($url, 'param');
+        }
+        break;
+    }
+
+    return $url;
+}
+
 @define('AUTH_HANDLER', true);
 @define('HORDE_BASE', dirname(__FILE__));
 require_once HORDE_BASE . '/lib/base.php';
-require_once 'Horde/Menu.php';
 require_once 'Horde/Secret.php';
 
 /* Initialize the Auth credentials key. */
@@ -26,18 +49,43 @@ if (is_a($auth, 'PEAR_Error')) {
 
 /* Get parameters. */
 $logout_reason = $auth->getLogoutReason();
-$url_param = Util::getFormData('url');
 $ie_version = Util::getFormData('ie_version');
 
+/* Get URL/Anchor strings now. */
+$url_anchor = null;
+$url_in = $url_form = Util::getFormData('url');
+if (($pos = strrpos($url_in, '#')) !== false) {
+    $url_anchor = substr($url_in, $pos + 1);
+    $url_in = substr($url_in, 0, $pos);
+}
+
 if ($logout_reason) {
+    if (Auth::getAuth()) {
+        $result = Horde::checkRequestToken('horde.logout', Util::getFormData('horde_logout_token'));
+        if (is_a($result, 'PEAR_Error')) {
+            exit($result->getMessage());
+        }
+    }
+
     $login_screen = $auth->getLoginScreen();
+    if (Util::getFormData('nosidebar') &&
+        isset($GLOBALS['notification'])) {
+        $url = Auth::addLogoutParameters($login_screen);
+        if (!empty($url_in)) {
+            $url = Util::addParameter($url, 'url', _addAnchor($url_in, 'param'));
+        }
+        $notification->push('window.parent.location.href = \'' . _addAnchor($url, 'url') . '\';', 'javascript');
+        echo '<html><body>' . $GLOBALS['notification']->notify(array('listeners' => array('javascript'))) . '</body></html>';
+        exit;
+    }
+
     if (Util::removeParameter($login_screen, array('url', 'nocache')) !=
         Util::removeParameter(Horde::selfUrl(false, false, true), array('url', 'nocache'))) {
         $url = Auth::addLogoutParameters($login_screen);
-        if ($url_param) {
-            $url = Util::addParameter($url, 'url', $url_param);
+        if ($url_in) {
+            $url = Util::addParameter($url, 'url', _addAnchor($url_in, 'param'));
         }
-        header('Location: ' . $url);
+        header('Location: ' . _addAnchor($url, 'url'));
         exit;
     }
 
@@ -49,12 +97,15 @@ if ($logout_reason) {
     Auth::clearAuth();
     @session_destroy();
 
-    /* FIXME: Add a redirect-on-logout setting and check for that
-     * here. */
-    // if () {
-    //     header('Location: ' . Horde::applicationUrl());
-    //     exit;
-    // }
+    /* Redirect the user on logout if redirection is enabled. */
+    if (!empty($conf['auth']['redirect_on_logout'])) {
+        $logout_url = $conf['auth']['redirect_on_logout'];
+        if (!isset($_COOKIE[session_name()])) {
+            $logout_url = Util::addParameter($logout_url, session_name(), session_id());
+        }
+        header('Location: ' . _addAnchor($logout_url, 'url'));
+        exit;
+    }
 
     Horde::setupSessionHandler();
     @session_start();
@@ -63,6 +114,17 @@ if ($logout_reason) {
 
     /* Hook to preselect the correct language in the widget. */
     $_GET['new_lang'] = $language;
+
+    if ($conf['menu']['always']) {
+        $main_page = Util::addParameter(Horde::selfUrl(), 'reason', $auth->getLogoutReasonString());
+        if ($browser->hasQuirk('scrollbar_in_way')) {
+            $scrollbar = 'yes';
+        } else {
+            $scrollbar = 'auto';
+        }
+        require HORDE_TEMPLATES . '/index/frames_index.inc';
+        exit;
+    }
 }
 
 if (isset($_POST['horde_user']) && isset($_POST['horde_pass'])) {
@@ -79,15 +141,15 @@ if (isset($_POST['horde_user']) && isset($_POST['horde_pass'])) {
             $browser->setIEVersion($ie_version);
         }
 
-        if ($url_param) {
-            $url = Horde::url(Util::removeParameter($url_param, session_name()), true);
+        if (!empty($url_in)) {
+            $url = Horde::url(Util::removeParameter($url_in, session_name()), true);
             $horde_url = $registry->get('webroot', 'horde') . '/index.php';
-            $horde_url = Util::addParameter($horde_url, 'url', $url);
+            $horde_url = Util::addParameter($horde_url, 'url', _addAnchor($url, 'param'));
         } else {
-            $horde_url = 'index.php';
+            $horde_url = Horde::url($registry->get('webroot', 'horde') . '/index.php');
         }
 
-        $url = Horde::applicationUrl($horde_url, true);
+        $url = _addAnchor(Horde::applicationUrl($horde_url, true), 'url');
         if ($browser->isBrowser('msie') &&
             $conf['use_ssl'] == 3 &&
             strlen($url) < 160) {
@@ -129,16 +191,29 @@ if (Auth::isAuthenticated()) {
     exit;
 }
 
+/* Redirect the user if an alternate login page has been specified. */
+if (!empty($conf['auth']['alternate_login'])) {
+    $url = Auth::addLogoutParameters($conf['auth']['alternate_login']);
+    if (!isset($_COOKIE[session_name()])) {
+        $url = Util::addParameter($url, session_name(), session_id(), false);
+    }
+    if (!empty($url_in)) {
+        $url = Util::addParameter($url, 'url', _addAnchor($url_in, 'param'), false);
+    }
+    header('Location: ' . _addAnchor($url, 'url'));
+    exit;
+}
+
 $login_screen = $auth->_getLoginScreen();
 if (Util::removeParameter($login_screen, array('url', 'nocache')) !=
     Horde::selfUrl(false, false, true)) {
-    if ($url_param) {
-        $login_screen = Util::addParameter($login_screen, 'url', $url_param);
+    if (!empty($url_in)) {
+        $login_screen = Util::addParameter($login_screen, 'url', _addAnchor($url_in, 'param'), false);
     }
     if ($ie_version) {
-        $login_screen = Util::addParameter($login_screen, 'ie_version', $ie_version);
+        $login_screen = Util::addParameter($login_screen, 'ie_version', $ie_version, false);
     }
-    header('Location: ' . $login_screen);
+    header('Location: ' . _addAnchor($login_screen, 'url'));
     exit;
 }
 
@@ -151,7 +226,7 @@ if ($browser->isMobile()) {
 /* Build the <select> widget containing the available languages. */
 if (!$prefs->isLocked('language')) {
     $_SESSION['horde_language'] = NLS::select();
-    $langs = '<select name="new_lang" onchange="selectLang()">';
+    $langs = '<select id="new_lang" name="new_lang" onchange="selectLang()">';
     foreach ($nls['languages'] as $key => $val) {
         $sel = ($key == $_SESSION['horde_language']) ? ' selected="selected"' : '';
         $langs .= "<option value=\"$key\"$sel>$val</option>";
@@ -165,10 +240,9 @@ if ($logout_reason && $conf['menu']['always']) {
     $notification->push('if (window.parent.frames.horde_menu) window.parent.frames.horde_menu.location.href = \'' . Horde::applicationUrl('services/portal/sidebar.php') . '\';', 'javascript');
 }
 
-$reason = $auth->getLogoutReasonString();
-
-/* Add some javascript. */
-Horde::addScriptFile('enter_key_trap.js', 'horde', true);
+if ($reason = $auth->getLogoutReasonString()) {
+    $notification->push(str_replace('<br />', ' ', $reason), 'horde.message');
+}
 
 /* Do we need to do IE version detection? */
 if (($browser->getBrowser() == 'msie') && ($browser->getMajor() >= 5)) {
