@@ -1,4 +1,4 @@
--- $Horde: horde/scripts/sql/horde_log.mssql.sql,v 1.1.2.1 2006/01/18 19:58:48 ben Exp $
+-- $Horde: horde/scripts/sql/horde_log.mssql.sql,v 1.1.2.2 2007/12/20 15:03:03 jan Exp $
 
 CREATE TABLE horde_log (
     id          INT NOT NULL,
@@ -10,5 +10,3 @@ CREATE TABLE horde_log (
     message     VARCHAR(MAX),
     PRIMARY KEY (id)
 );
-
-GRANT INSERT ON horde_log TO horde;
